@@ -17,7 +17,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: idlist.c,v 1.76 2002/09/16 10:21:28 priikone Exp $ */
+/* $Id: idlist.c,v 1.79.2.4 2002/12/16 18:53:53 priikone Exp $ */
 
 #include "serverincludes.h"
 #include "idlist.h"
@@ -227,7 +227,7 @@ silc_idlist_find_server_by_conn(SilcIDList id_list, char *hostname,
     
     if (sock && ((sock->hostname && !strcasecmp(sock->hostname, hostname)) ||
 		 (sock->ip && !strcasecmp(sock->ip, hostname)))
-	&& sock->port == port)
+	&& server->id->port == htons(port))
       break;
 
     id_cache = NULL;
@@ -295,8 +295,10 @@ int silc_idlist_del_server(SilcIDList id_list, SilcServerEntry entry)
     if (!silc_idcache_del_by_context(id_list->servers, entry))
       return FALSE;
 
-    SILC_LOG_DEBUG(("Deleting server %s", entry->server_name ?
-		    entry->server_name : ""));
+    SILC_LOG_DEBUG(("Deleting server %s id %s", entry->server_name ?
+		    entry->server_name : "",
+		    entry->id ?
+		    silc_id_render(entry->id, SILC_ID_SERVER) : ""));
 
     /* Free data */
     silc_free(entry->server_name);
@@ -376,6 +378,7 @@ int silc_idlist_del_client(SilcIDList id_list, SilcClientEntry entry)
     silc_free(entry->username);
     silc_free(entry->userinfo);
     silc_free(entry->id);
+    silc_free(entry->attrs);
     silc_hash_table_free(entry->channels);
 
     memset(entry, 'F', sizeof(*entry));
@@ -575,6 +578,7 @@ void silc_idlist_client_destructor(SilcIDCache cache,
     silc_free(client->username);
     silc_free(client->userinfo);
     silc_free(client->id);
+    silc_free(client->attrs);
     silc_hash_table_free(client->channels);
 
     memset(client, 'A', sizeof(*client));
@@ -669,6 +673,12 @@ int silc_idlist_del_channel(SilcIDList id_list, SilcChannelEntry entry)
     silc_free(entry->channel_name);
     silc_free(entry->id);
     silc_free(entry->topic);
+
+    if (entry->invite_list)
+      silc_hash_table_free(entry->invite_list);
+    if (entry->ban_list)
+      silc_hash_table_free(entry->ban_list);
+
     if (entry->channel_key)
       silc_cipher_free(entry->channel_key);
     if (entry->key) {
